@@ -1,57 +1,110 @@
 /*
-    SPDX-FileCopyrightText: 2005 Aaron Seigo <aseigo@kde.org>
-    SPDX-FileCopyrightText: 2007 Riccardo Iaconelli <riccardo@kde.org>
-    SPDX-FileCopyrightText: 2008 Ménard Alexis <darktears31@gmail.com>
-    SPDX-FileCopyrightText: 2009 Chani Armitage <chani@kde.org>
-
-    SPDX-License-Identifier: LGPL-2.0-or-later
-*/
+ *   Copyright 2005 by Aaron Seigo <aseigo@kde.org>
+ *   Copyright 2007 by Riccardo Iaconelli <riccardo@kde.org>
+ *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
+ *   Copyright (c) 2009 Chani Armitage <chani@kde.org>
+ *
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU Library General Public License as
+ *   published by the Free Software Foundation; either version 2, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details
+ *
+ *   You should have received a copy of the GNU Library General Public
+ *   License along with this program; if not, write to the
+ *   Free Software Foundation, Inc.,
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ */
 
 #include "applet.h"
 #include "private/applet_p.h"
 
 #include "config-plasma.h"
 
-#include <QAbstractButton>
-#include <QActionGroup>
-#include <QDebug>
-#include <QFile>
-#include <QJSEngine>
-#include <QList>
-#include <QMessageBox>
-#include <QMetaEnum>
+#include <cmath>
+#include <limits>
 
-#include <KAuthorized>
-#include <KColorScheme>
+#include <QFile>
+#include <QList>
+#include <QAbstractButton>
+#include <QMessageBox>
+
+#include <kactioncollection.h>
+#include <kauthorized.h>
+#include <kcolorscheme.h>
+#include <kdesktopfile.h>
+#include <QDebug>
+#include <kglobalaccel.h>
+#include <kplugininfo.h>
+#include <klocalizedstring.h>
+#include <kservice.h>
 #include <KConfigLoader>
-#include <KConfigPropertyMap>
-#include <KDesktopFile>
-#include <KGlobalAccel>
-#include <KLocalizedString>
-#include <KPackage/Package>
+#include <kwindowsystem.h>
 
 #include "containment.h"
 #include "corona.h"
+#include "package.h"
 #include "plasma.h"
+#include "scripting/appletscript.h"
 #include "pluginloader.h"
 
-#include "debug_p.h"
+#include "private/associatedapplicationmanager_p.h"
 #include "private/containment_p.h"
-
-#include <cmath>
-#include <limits>
-#include <qaction.h>
+#include "private/package_p.h"
+#include "debug_p.h"
 
 namespace Plasma
 {
-Applet::Applet(QObject *parentObject, const KPluginMetaData &data, const QVariantList &args)
-    : QObject(parentObject)
-    , d(new AppletPrivate(data, args.count() > 1 ? args[1].toInt() : 0, this))
+
+static KPluginMetaData appletMetadataForDirectory(const QString &path)
 {
-    if (!args.isEmpty()) {
+    return QFile::exists(path + QLatin1String("/metadata.json"))
+                ? KPluginMetaData(path + QLatin1String("/metadata.json"))
+                : KPluginMetaData::fromDesktopFile(path + QLatin1String("/metadata.desktop"), { QStringLiteral("plasma-applet.desktop") });
+}
+
+Applet::Applet(const KPluginMetaData &info, QObject *parent, uint appletId)
+    :  QObject(parent),
+       d(new AppletPrivate(info, appletId, this))
+{
+    qCDebug(LOG_PLASMA) << " From KPluginMetaData, valid? " << info.isValid();
+    // WARNING: do not access config() OR globalConfig() in this method!
+    //          that requires a scene, which is not available at this point
+    d->init();
+    d->setupPackage();
+}
+
+Applet::Applet(const KPluginInfo &info, QObject *parent, uint appletId)
+    :  Applet(info.toMetaData(), parent, appletId)
+{
+}
+
+Applet::Applet(QObject *parent, const QString &serviceID, uint appletId)
+    :  QObject(parent),
+       d(new AppletPrivate(KPluginMetaData(serviceID), appletId, this))
+{
+    // WARNING: do not access config() OR globalConfig() in this method!
+    //          that requires a scene, which is not available at this point
+    d->init();
+    d->setupPackage();
+}
+
+Applet::Applet(QObject *parentObject, const QVariantList &args)
+    :  QObject(0),
+       d(new AppletPrivate(KPluginMetaData(), args.count() > 1 ? args[1].toInt() : 0, this))
+{
+    setParent(parentObject);
+    if (args.count() > 0) {
         const QVariant first = args.first();
-        if (first.canConvert<KPackage::Package>()) {
-            d->package = first.value<KPackage::Package>();
+        if (first.canConvert<QString>()) {
+            d->appletDescription = KPluginMetaData(first.toString());
+        } else if (first.canConvert<QVariantMap>()) {
+            auto metadata = first.toMap().value(QStringLiteral("MetaData")).toMap();
+            d->appletDescription = KPluginMetaData(QJsonObject::fromVariantMap(metadata), {});
         }
     }
     d->icon = d->appletDescription.iconName();
@@ -62,36 +115,32 @@ Applet::Applet(QObject *parentObject, const KPluginMetaData &data, const QVarian
 
     // WARNING: do not access config() OR globalConfig() in this method!
     //          that requires a scene, which is not available at this point
-    d->init(args.mid(2));
+    d->init(QString(), args.mid(2));
+    d->setupPackage();
+}
+
+Applet::Applet(const QString &packagePath, uint appletId)
+    : QObject(0),
+      d(new AppletPrivate(appletMetadataForDirectory(packagePath), appletId, this))
+{
+    d->init(packagePath);
     d->setupPackage();
 }
 
 Applet::~Applet()
 {
-    for (QAction *a : d->actions.values()) {
-        disconnect(a, nullptr, this, nullptr);
-    }
-    for (QAction *a : d->contextualActions) {
-        disconnect(a, nullptr, this, nullptr);
-    }
     if (d->transient) {
         d->resetConfigurationObject();
     }
-    // let people know that i will die
-    Q_EMIT appletDeleted(this);
+    //let people know that i will die
+    emit appletDeleted(this);
 
-    // ConfigLoader is deleted when AppletPrivate closes not Applet
-    // It saves on closure and emits a signal.
-    // disconnect early to avoid a crash. See  411221
-    if (d->configLoader) {
-        disconnect(d->configLoader, SIGNAL(configChanged()), this, SLOT(propagateConfigChanged()));
-    }
     delete d;
 }
 
 void Applet::init()
 {
-    // Don't implement anything here, it will be overridden by subclasses
+    //Don't implement anything here, it will be overridden by subclasses
 }
 
 uint Applet::id() const
@@ -99,11 +148,6 @@ uint Applet::id() const
     return d->appletId;
 }
 
-QVariantList Applet::startupArguments() const
-{
-    return d->startupArguments;
-}
-
 void Applet::save(KConfigGroup &g) const
 {
     if (d->transient || !d->appletDescription.isValid()) {
@@ -115,7 +159,7 @@ void Applet::save(KConfigGroup &g) const
         group = *d->mainConfigGroup();
     }
 
-    // qCDebug(LOG_PLASMA) << "saving" << pluginName() << "to" << group.name();
+    //qCDebug(LOG_PLASMA) << "saving" << pluginName() << "to" << group.name();
     // we call the dptr member directly for locked since isImmutable()
     // also checks kiosk and parent containers
     group.writeEntry("immutability", (int)d->immutability);
@@ -125,7 +169,7 @@ void Applet::save(KConfigGroup &g) const
         return;
     }
 
-    KConfigGroup appletConfigGroup(&group, QStringLiteral("Configuration"));
+    KConfigGroup appletConfigGroup(&group, "Configuration");
     saveState(appletConfigGroup);
 
     if (d->configLoader) {
@@ -139,9 +183,10 @@ void Applet::save(KConfigGroup &g) const
 
 void Applet::restore(KConfigGroup &group)
 {
+
     setImmutability((Types::ImmutabilityType)group.readEntry("immutability", (int)Types::Mutable));
 
-    KConfigGroup shortcutConfig(&group, QStringLiteral("Shortcuts"));
+    KConfigGroup shortcutConfig(&group, "Shortcuts");
     QString shortcutText = shortcutConfig.readEntryUntranslated("global", QString());
     if (!shortcutText.isEmpty()) {
         setGlobalShortcut(QKeySequence(shortcutText));
@@ -156,20 +201,18 @@ void Applet::restore(KConfigGroup &group)
                  */
     }
 
-    // User background hints
-    // TODO support flags in the config
-    QByteArray hintsString = config().readEntry("UserBackgroundHints", QString()).toUtf8();
-    QMetaEnum hintEnum = QMetaEnum::fromType<Plasma::Types::BackgroundHints>();
-    bool ok;
-    int value = hintEnum.keyToValue(hintsString.constData(), &ok);
-    if (ok) {
-        d->userBackgroundHints = Plasma::Types::BackgroundHints(value);
-        d->userBackgroundHintsInitialized = true;
-        Q_EMIT userBackgroundHintsChanged();
-        if (d->backgroundHints & Plasma::Types::ConfigurableBackground) {
-            Q_EMIT effectiveBackgroundHintsChanged();
-        }
+    // local shortcut, if any
+    //TODO: implement; the shortcut will need to be registered with the containment
+    /*
+    #include "accessmanager.h"
+    #include "private/plasmoidservice_p.h"
+    #include "authorizationmanager.h"
+    #include "authorizationmanager.h"
+    shortcutText = shortcutConfig.readEntryUntranslated("local", QString());
+    if (!shortcutText.isEmpty()) {
+        //TODO: implement; the shortcut
     }
+    */
 }
 
 void Applet::setLaunchErrorMessage(const QString &message)
@@ -184,6 +227,10 @@ void Applet::setLaunchErrorMessage(const QString &message)
 
 void Applet::saveState(KConfigGroup &group) const
 {
+    if (d->script) {
+        emit d->script->saveState(group);
+    }
+
     if (group.config()->name() != config().config()->name()) {
         // we're being saved to a different file!
         // let's just copy the current values in our configuration over
@@ -195,14 +242,14 @@ void Applet::saveState(KConfigGroup &group) const
 KConfigGroup Applet::config() const
 {
     if (d->transient) {
-        return KConfigGroup(KSharedConfig::openConfig(), QStringLiteral("PlasmaTransientsConfig"));
+        return KConfigGroup(KSharedConfig::openConfig(), "PlasmaTransientsConfig");
     }
 
     if (isContainment()) {
         return *(d->mainConfigGroup());
     }
 
-    return KConfigGroup(d->mainConfigGroup(), QStringLiteral("Configuration"));
+    return KConfigGroup(d->mainConfigGroup(), "Configuration");
 }
 
 KConfigGroup Applet::globalConfig() const
@@ -211,7 +258,7 @@ KConfigGroup Applet::globalConfig() const
     QString group = isContainment() ? QStringLiteral("ContainmentGlobals") : QStringLiteral("AppletGlobals");
 
     Containment *cont = containment();
-    Corona *corona = nullptr;
+    Corona *corona = 0;
     if (cont) {
         corona = cont->corona();
     }
@@ -228,11 +275,11 @@ KConfigGroup Applet::globalConfig() const
 void Applet::destroy()
 {
     if (immutability() != Types::Mutable || d->transient || !d->started) {
-        return; // don't double delete
+        return; //don't double delete
     }
 
     d->setDestroyed(true);
-    // FIXME: an animation on leave if !isContainment() would be good again .. which should be handled by the containment class
+    //FIXME: an animation on leave if !isContainment() would be good again .. which should be handled by the containment class
     d->cleanUpAndDelete();
 }
 
@@ -244,10 +291,10 @@ bool Applet::destroyed() const
 KConfigLoader *Applet::configScheme() const
 {
     if (!d->configLoader) {
-        const QString xmlPath = d->package.isValid() ? d->package.filePath("mainconfigxml") : QString();
+        const QString xmlPath = d->package ? d->package->filePath("mainconfigxml") : QString();
         KConfigGroup cfg = config();
         if (xmlPath.isEmpty()) {
-            d->configLoader = new KConfigLoader(cfg, nullptr);
+            d->configLoader = new KConfigLoader(cfg, 0);
         } else {
             QFile file(xmlPath);
             d->configLoader = new KConfigLoader(cfg, &file);
@@ -258,33 +305,37 @@ KConfigLoader *Applet::configScheme() const
     return d->configLoader;
 }
 
-KConfigPropertyMap *Applet::configuration()
+Package Applet::package() const
 {
-    if (!d->configPropertyMap) {
-        d->configPropertyMap = new KConfigPropertyMap(configScheme(), this);
+    Package p;
+    if (d->package) {
+        p.d->internalPackage = new KPackage::Package(*d->package);
     }
-    return d->configPropertyMap;
+    return p;
 }
 
 KPackage::Package Applet::kPackage() const
 {
-    return d->package;
+    return d->package ? *d->package : KPackage::Package();
 }
 
-void Applet::updateConstraints(Constraints constraints)
+void Applet::updateConstraints(Plasma::Types::Constraints constraints)
 {
     d->scheduleConstraintsUpdate(constraints);
 }
 
-void Applet::constraintsEvent(Constraints constraints)
+void Applet::constraintsEvent(Plasma::Types::Constraints constraints)
 {
-    // NOTE: do NOT put any code in here that reacts to constraints updates
+    //NOTE: do NOT put any code in here that reacts to constraints updates
     //      as it will not get called for any applet that reimplements constraintsEvent
     //      without calling the Applet:: version as well, which it shouldn't need to.
     //      INSTEAD put such code into flushPendingConstraintsEvents
     Q_UNUSED(constraints)
-    // qCDebug(LOG_PLASMA) << constraints << "constraints are FormFactor: " << formFactor()
+    //qCDebug(LOG_PLASMA) << constraints << "constraints are FormFactor: " << formFactor()
     //         << ", Location: " << location();
+    if (d->script) {
+        d->script->constraintsEvent(constraints);
+    }
 }
 
 QString Applet::title() const
@@ -307,7 +358,7 @@ void Applet::setTitle(const QString &title)
     }
 
     d->customTitle = title;
-    Q_EMIT titleChanged(title);
+    emit titleChanged(title);
 }
 
 QString Applet::icon() const
@@ -322,7 +373,7 @@ void Applet::setIcon(const QString &icon)
     }
 
     d->icon = icon;
-    Q_EMIT iconChanged(icon);
+    emit iconChanged(icon);
 }
 
 bool Applet::isBusy() const
@@ -337,63 +388,12 @@ void Applet::setBusy(bool busy)
     }
 
     d->busy = busy;
-    Q_EMIT busyChanged(busy);
+    emit busyChanged(busy);
 }
 
-Plasma::Types::BackgroundHints Applet::backgroundHints() const
+KPluginInfo Applet::pluginInfo() const
 {
-    return d->backgroundHints;
-}
-
-void Applet::setBackgroundHints(Plasma::Types::BackgroundHints hint)
-{
-    if (d->backgroundHints == hint) {
-        return;
-    }
-
-    Plasma::Types::BackgroundHints oldeffectiveHints = effectiveBackgroundHints();
-
-    d->backgroundHints = hint;
-    Q_EMIT backgroundHintsChanged();
-
-    if (oldeffectiveHints != effectiveBackgroundHints()) {
-        Q_EMIT effectiveBackgroundHintsChanged();
-    }
-}
-
-Plasma::Types::BackgroundHints Applet::effectiveBackgroundHints() const
-{
-    if (d->userBackgroundHintsInitialized && (d->backgroundHints & Plasma::Types::ConfigurableBackground)) {
-        return d->userBackgroundHints;
-    } else {
-        return d->backgroundHints;
-    }
-}
-
-Plasma::Types::BackgroundHints Applet::userBackgroundHints() const
-{
-    return d->userBackgroundHints;
-}
-
-void Applet::setUserBackgroundHints(Plasma::Types::BackgroundHints hint)
-{
-    if (d->userBackgroundHints == hint && d->userBackgroundHintsInitialized) {
-        return;
-    }
-
-    d->userBackgroundHints = hint;
-    d->userBackgroundHintsInitialized = true;
-    QMetaEnum hintEnum = QMetaEnum::fromType<Plasma::Types::BackgroundHints>();
-    config().writeEntry("UserBackgroundHints", hintEnum.valueToKey(d->userBackgroundHints));
-    if (containment() && containment()->corona()) {
-        containment()->corona()->requestConfigSync();
-    }
-
-    Q_EMIT userBackgroundHintsChanged();
-
-    if (d->backgroundHints & Plasma::Types::ConfigurableBackground) {
-        Q_EMIT effectiveBackgroundHintsChanged();
-    }
+    return KPluginInfo(d->appletDescription);
 }
 
 KPluginMetaData Applet::pluginMetaData() const
@@ -401,11 +401,6 @@ KPluginMetaData Applet::pluginMetaData() const
     return d->appletDescription;
 }
 
-QString Applet::pluginName() const
-{
-    return d->appletDescription.isValid() ? d->appletDescription.pluginId() : QString();
-}
-
 Types::ImmutabilityType Applet::immutability() const
 {
     // if this object is itself system immutable, then just return that; it's the most
@@ -415,7 +410,7 @@ Types::ImmutabilityType Applet::immutability() const
         return Types::SystemImmutable;
     }
 
-    // Returning the more strict immutability between the applet immutability, Containment and Corona
+    //Returning the more strict immutability between the applet immutability, Containment and Corona
     Types::ImmutabilityType upperImmutability = Types::Mutable;
 
     if (isContainment()) {
@@ -454,12 +449,7 @@ void Applet::setImmutability(const Types::ImmutabilityType immutable)
     }
 
     d->immutability = immutable;
-    updateConstraints(ImmutableConstraint);
-}
-
-bool Applet::immutable() const
-{
-    return immutability() != Types::Mutable;
+    updateConstraints(Types::ImmutableConstraint);
 }
 
 QString Applet::launchErrorMessage() const
@@ -491,22 +481,7 @@ void Applet::setConfigurationRequired(bool needsConfig, const QString &reason)
     d->needsConfig = needsConfig;
     d->configurationRequiredReason = reason;
 
-    Q_EMIT configurationRequiredChanged(needsConfig, reason);
-}
-
-void Applet::setConstraintHints(ConstraintHints constraintHints)
-{
-    if (d->constraintHints == constraintHints) {
-        return;
-    }
-
-    d->constraintHints = constraintHints;
-    Q_EMIT constraintHintsChanged(constraintHints);
-}
-
-Applet::ConstraintHints Applet::constraintHints() const
-{
-    return d->constraintHints;
+    emit configurationRequiredChanged(needsConfig, reason);
 }
 
 bool Applet::isUserConfiguring() const
@@ -521,7 +496,7 @@ void Applet::setUserConfiguring(bool configuring)
     }
 
     d->userConfiguring = configuring;
-    Q_EMIT userConfiguringChanged(configuring);
+    emit userConfiguringChanged(configuring);
 }
 
 Types::ItemStatus Applet::status() const
@@ -535,12 +510,12 @@ void Applet::setStatus(const Types::ItemStatus status)
         return;
     }
     d->itemStatus = status;
-    Q_EMIT statusChanged(status);
+    emit statusChanged(status);
 }
 
 void Applet::flushPendingConstraintsEvents()
 {
-    if (d->pendingConstraints == NoConstraint) {
+    if (d->pendingConstraints == Types::NoConstraint) {
         return;
     }
 
@@ -548,58 +523,66 @@ void Applet::flushPendingConstraintsEvents()
         d->constraintsTimer.stop();
     }
 
-    // qCDebug(LOG_PLASMA) << "flushing constraints: " << d->pendingConstraints << "!!!!!!!!!!!!!!!!!!!!!!!!!!!";
-    Constraints c = d->pendingConstraints;
-    d->pendingConstraints = NoConstraint;
+    //qCDebug(LOG_PLASMA) << "fushing constraints: " << d->pendingConstraints << "!!!!!!!!!!!!!!!!!!!!!!!!!!!";
+    Plasma::Types::Constraints c = d->pendingConstraints;
+    d->pendingConstraints = Types::NoConstraint;
 
-    if (c & UiReadyConstraint) {
+    if (c & Plasma::Types::UiReadyConstraint) {
         d->setUiReady();
     }
 
-    if (c & StartupCompletedConstraint) {
-        // common actions
+    if (c & Plasma::Types::StartupCompletedConstraint) {
+        //common actions
         bool unlocked = immutability() == Types::Mutable;
-        QAction *closeApplet = d->actions.value(QStringLiteral("remove"));
+        QAction *closeApplet = d->actions->action(QStringLiteral("remove"));
         if (closeApplet) {
             closeApplet->setEnabled(unlocked);
             closeApplet->setVisible(unlocked);
             connect(closeApplet, SIGNAL(triggered(bool)), this, SLOT(askDestroy()), Qt::UniqueConnection);
         }
 
-        QAction *configAction = d->actions.value(QStringLiteral("configure"));
+        QAction *configAction = d->actions->action(QStringLiteral("configure"));
         if (configAction) {
+
             if (d->hasConfigurationInterface) {
                 bool canConfig = unlocked || KAuthorized::authorize(QStringLiteral("plasma/allow_configure_when_locked"));
                 configAction->setVisible(canConfig);
                 configAction->setEnabled(canConfig);
             }
         }
+
+        QAction *runAssociatedApplication = d->actions->action(QStringLiteral("run associated application"));
+        if (runAssociatedApplication) {
+            connect(runAssociatedApplication, SIGNAL(triggered(bool)), this, SLOT(runAssociatedApplication()), Qt::UniqueConnection);
+        }
+
+        d->updateShortcuts();
     }
 
-    if (c & ImmutableConstraint) {
+    if (c & Plasma::Types::ImmutableConstraint) {
         bool unlocked = immutability() == Types::Mutable;
-        QAction *action = d->actions.value(QStringLiteral("remove"));
+        QAction *action = d->actions->action(QStringLiteral("remove"));
         if (action) {
             action->setVisible(unlocked);
             action->setEnabled(unlocked);
         }
 
-        action = d->actions.value(QStringLiteral("configure"));
+        action = d->actions->action(QStringLiteral("configure"));
         if (action && d->hasConfigurationInterface) {
             bool canConfig = unlocked || KAuthorized::authorize(QStringLiteral("plasma/allow_configure_when_locked"));
             action->setVisible(canConfig);
             action->setEnabled(canConfig);
         }
 
-        // an immutable constraint will always happen at startup
-        // make sure don't emit a change signal for nothing
+        //an immutable constraint will alwasy happen at startup
+        //make sure don't emit a change signal for nothing
         if (d->oldImmutability != immutability()) {
-            Q_EMIT immutabilityChanged(immutability());
+            emit immutabilityChanged(immutability());
         }
         d->oldImmutability = immutability();
     }
 
-    // now take care of constraints in special subclass: Containment
+    // now take care of constraints in special subclass: Contaiment
     Containment *containment = qobject_cast<Plasma::Containment *>(this);
     if (containment) {
         containment->d->containmentConstraintsEvent(c);
@@ -608,7 +591,7 @@ void Applet::flushPendingConstraintsEvents()
     // pass the constraint on to the actual subclass
     constraintsEvent(c);
 
-    if (c & StartupCompletedConstraint) {
+    if (c & Types::StartupCompletedConstraint) {
         // start up is done, we can now go do a mod timer
         if (d->modificationsTimer) {
             if (d->modificationsTimer->isActive()) {
@@ -619,76 +602,24 @@ void Applet::flushPendingConstraintsEvents()
         }
     }
 
-    if (c & FormFactorConstraint) {
-        Q_EMIT formFactorChanged(formFactor());
+    if (c & Plasma::Types::FormFactorConstraint) {
+        emit formFactorChanged(formFactor());
     }
 
-    if (c & LocationConstraint) {
-        Q_EMIT locationChanged(location());
+    if (c & Plasma::Types::LocationConstraint) {
+        emit locationChanged(location());
     }
 }
 
 QList<QAction *> Applet::contextualActions()
 {
-    return d->contextualActions;
+    //qCDebug(LOG_PLASMA) << "empty context actions";
+    return d->script ? d->script->contextualActions() : QList<QAction *>();
 }
 
-QQmlListProperty<QAction> Applet::qmlContextualActions()
+KActionCollection *Applet::actions() const
 {
-    return QQmlListProperty<QAction>(this,
-                                     nullptr,
-                                     AppletPrivate::contextualActions_append,
-                                     AppletPrivate::contextualActions_count,
-                                     AppletPrivate::contextualActions_at,
-                                     AppletPrivate::contextualActions_clear,
-                                     AppletPrivate::contextualActions_replace,
-                                     AppletPrivate::contextualActions_removeLast);
-}
-
-void Applet::setInternalAction(const QString &name, QAction *action)
-{
-    if (name.isEmpty()) {
-        return;
-    }
-
-    action->setObjectName(name);
-    QAction *oldAction = d->actions.value(name);
-    if (oldAction && QJSEngine::objectOwnership(oldAction) == QJSEngine::CppOwnership) {
-        delete oldAction;
-    }
-
-    d->actions[name] = action;
-
-    QObject::connect(action, &QObject::destroyed, this, [this, name]() {
-        d->actions.remove(name);
-        Q_EMIT internalActionsChanged(d->actions.values());
-    });
-
-    Q_EMIT internalActionsChanged(d->actions.values());
-}
-
-QAction *Applet::internalAction(const QString &name) const
-{
-    return d->actions.value(name);
-}
-
-void Applet::removeInternalAction(const QString &name)
-{
-    QAction *action = d->actions.value(name);
-
-    if (action && QJSEngine::objectOwnership(action) == QJSEngine::CppOwnership) {
-        disconnect(action, &QObject::destroyed, this, nullptr); // Avoid emitting signal again
-        delete action;
-    }
-
-    d->actions.remove(name);
-
-    Q_EMIT internalActionsChanged(d->actions.values());
-}
-
-QList<QAction *> Applet::internalActions() const
-{
-    return d->actions.values();
+    return d->actions;
 }
 
 Types::FormFactor Applet::formFactor() const
@@ -696,7 +627,7 @@ Types::FormFactor Applet::formFactor() const
     Containment *c = containment();
     QObject *pw = qobject_cast<QObject *>(parent());
     Plasma::Applet *parentApplet = qobject_cast<Plasma::Applet *>(pw);
-    // assumption: this loop is usually is -really- short or doesn't run at all
+    //assumption: this loop is usually is -really- short or doesn't run at all
     while (!parentApplet && pw && pw->parent()) {
         pw = pw->parent();
         parentApplet = qobject_cast<Plasma::Applet *>(pw);
@@ -705,20 +636,13 @@ Types::FormFactor Applet::formFactor() const
     return c ? c->d->formFactor : Plasma::Types::Planar;
 }
 
-Types::ContainmentDisplayHints Applet::containmentDisplayHints() const
-{
-    Containment *c = containment();
-
-    return c ? c->d->containmentDisplayHints : Plasma::Types::NoContainmentDisplayHint;
-}
-
 Containment *Applet::containment() const
 {
     Containment *c = qobject_cast<Containment *>(const_cast<Applet *>(this));
     if (c && c->isContainment()) {
         return c;
     } else {
-        c = nullptr;
+        c = 0;
     }
 
     QObject *parent = this->parent();
@@ -742,24 +666,19 @@ void Applet::setGlobalShortcut(const QKeySequence &shortcut)
         d->activationAction = new QAction(this);
         d->activationAction->setText(i18n("Activate %1 Widget", title()));
         d->activationAction->setObjectName(QStringLiteral("activate widget %1").arg(id())); // NO I18N
-        connect(d->activationAction, &QAction::triggered, this, &Applet::activated);
-        connect(KGlobalAccel::self(), &KGlobalAccel::globalShortcutChanged, this, [this](QAction *action, const QKeySequence &shortcut) {
-            if (action == d->activationAction) {
-                d->activationAction->setShortcut(shortcut);
-                d->globalShortcutChanged();
-            }
-        });
+        connect(d->activationAction, SIGNAL(triggered()), this, SIGNAL(activated()));
+        connect(d->activationAction, SIGNAL(changed()),
+                this, SLOT(globalShortcutChanged()));
     } else if (d->activationAction->shortcut() == shortcut) {
         return;
     }
 
     d->activationAction->setShortcut(shortcut);
     d->globalShortcutEnabled = true;
-    QList<QKeySequence> seqs{shortcut};
+    QList<QKeySequence> seqs;
+    seqs << shortcut;
     KGlobalAccel::self()->setShortcut(d->activationAction, seqs, KGlobalAccel::NoAutoloading);
     d->globalShortcutChanged();
-
-    Q_EMIT globalShortcutChanged(shortcut);
 }
 
 QKeySequence Applet::globalShortcut() const
@@ -791,7 +710,7 @@ void Applet::setHasConfigurationInterface(bool hasInterface)
         return;
     }
 
-    QAction *configAction = d->actions.value(QStringLiteral("configure"));
+    QAction *configAction = d->actions->action(QStringLiteral("configure"));
     if (configAction) {
         bool enable = hasInterface;
         if (enable) {
@@ -806,17 +725,72 @@ void Applet::setHasConfigurationInterface(bool hasInterface)
 
 void Applet::configChanged()
 {
-    if (d->configLoader) {
-        d->configLoader->load();
+    if (d->script) {
+        if (d->configLoader) {
+            d->configLoader->load();
+        }
+        d->script->configChanged();
     }
 }
 
-QString Applet::filePath(const QByteArray &key, const QString &filename) const
+void Applet::setAssociatedApplication(const QString &string)
 {
-    if (d->package.isValid()) {
-        return d->package.filePath(key, filename);
+    AssociatedApplicationManager::self()->setApplication(this, string);
+
+    QAction *runAssociatedApplication = d->actions->action(QStringLiteral("run associated application"));
+    if (runAssociatedApplication) {
+        bool valid = AssociatedApplicationManager::self()->appletHasValidAssociatedApplication(this);
+        runAssociatedApplication->setVisible(valid);
+        runAssociatedApplication->setEnabled(valid);
     }
-    return QString();
+}
+
+void Applet::setAssociatedApplicationUrls(const QList<QUrl> &urls)
+{
+    AssociatedApplicationManager::self()->setUrls(this, urls);
+
+    QAction *runAssociatedApplication = d->actions->action(QStringLiteral("run associated application"));
+    if (runAssociatedApplication) {
+        bool valid = AssociatedApplicationManager::self()->appletHasValidAssociatedApplication(this);
+        runAssociatedApplication->setVisible(valid);
+        runAssociatedApplication->setEnabled(valid);
+    }
+}
+
+QString Applet::associatedApplication() const
+{
+    return AssociatedApplicationManager::self()->application(this);
+}
+
+QList<QUrl> Applet::associatedApplicationUrls() const
+{
+    return AssociatedApplicationManager::self()->urls(this);
+}
+
+void Applet::runAssociatedApplication()
+{
+    AssociatedApplicationManager::self()->run(this);
+}
+
+bool Applet::hasValidAssociatedApplication() const
+{
+    return AssociatedApplicationManager::self()->appletHasValidAssociatedApplication(this);
+}
+
+Applet *Applet::loadPlasmoid(const QString &path, uint appletId)
+{
+    const KPluginMetaData md = appletMetadataForDirectory(path);
+    if (md.isValid()) {
+        QStringList types = md.serviceTypes();
+
+        if (types.contains(QStringLiteral("Plasma/Containment"))) {
+            return new Containment(md, appletId);
+        } else {
+            return new Applet(md, nullptr, appletId);
+        }
+    }
+
+    return 0;
 }
 
 void Applet::timerEvent(QTimerEvent *event)
@@ -834,7 +808,7 @@ void Applet::timerEvent(QTimerEvent *event)
 
         // Don't flushPendingConstraints if we're just starting up
         // flushPendingConstraints will be called by Corona
-        if (!(d->pendingConstraints & StartupCompletedConstraint)) {
+        if (!(d->pendingConstraints & Plasma::Types::StartupCompletedConstraint)) {
             flushPendingConstraintsEvents();
         }
     } else if (d->modificationsTimer && event->timerId() == d->modificationsTimer->timerId()) {
@@ -843,19 +817,19 @@ void Applet::timerEvent(QTimerEvent *event)
         KConfigGroup cg;
 
         save(cg);
-        Q_EMIT configNeedsSaving();
+        emit configNeedsSaving();
     }
 }
 
 bool Applet::isContainment() const
 {
-    // HACK: this is a special case for the systray
-    // containment in an applet that is not a containment
+    //HACK: this is a special case for the systray
+    //containment in an applet that is not a containment
     Applet *pa = qobject_cast<Applet *>(parent());
     if (pa && !pa->isContainment()) {
         return true;
     }
-    // normal "acting as a containment" condition
+    //normal "acting as a containment" condition
     return qobject_cast<const Containment *>(this) && qobject_cast<Corona *>(parent());
 }
 
